/*  Dvortype is a typing tutor for the dvorak keyboard layout
    Copyright 2002 John Bauman
    ABCD Dvorak Lessons Copyright 1995-99 Dan Wood
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    John Bauman may be contacted by email at: baumans@ptdprolog.net
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <time.h>
#include <unistd.h>

#define PROGFILE "prog.dat"
#define LESSFILE "dvless.txt"


int numless;
int *linless;
char ***less;
char *curline;
int cless,cline;
time_t started;
int length;
int letters,errors;

/*
  Load Lessons from file
*/
void loadless() {
  FILE *f = fopen(LESSFILE,"r");
  char fromfile[100];
  int x = 0,y=0;
  fgets(fromfile,100,f);
  numless = atoi(fromfile);
  less = malloc(sizeof(char **) * numless);
  linless = malloc(sizeof(int) * numless);
  for (x = 0;x<numless;x++) {
   fgets(fromfile,100,f);
   linless[x] = atoi(fromfile);
   less[x] = malloc(sizeof(char*) * linless[x]);
   for (y = 0;y<linless[x];y++) {
    fgets(fromfile,100,f);
    fromfile[strlen(fromfile) - 1] = '\0';
    less[x][y] = strdup(fromfile);
   }
  }
  fclose(f);
}

GtkWidget *heading,*line,*window,*entry;

gint eventDelete(GtkWidget *widget, GdkEvent *event,gpointer data);
gint eventDestroy(GtkWidget *widget, GdkEvent *event,gpointer data);
gint tchanged(GtkWidget *widget, gpointer data);

int main(int argc,char **argv) {
  GtkWidget *box,*im;
  FILE *pro;
  GError *e = NULL;
  GdkPixbuf *gpb;
  char tstr[200];
  loadless();
  gtk_init(&argc,&argv);

  pro = fopen(PROGFILE,"r");
  if (pro) {
    fscanf(pro,"%d %d %d %d %d",&cless, &cline, &length, &errors, &letters);
    fclose(pro);
  } else {
    cless = 0;
    cline = 1;
    length = 0;
    errors = 0;
    letters = strlen(less[cless][cline]);
  }

  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  sprintf(tstr,"ABCD Dvorak - %d/%d %d/%d",cless,numless,cline,linless[cless]);
  gtk_window_set_title(GTK_WINDOW(window),tstr);

  gpb = gdk_pixbuf_new_from_file("key.png",&e);
  gtk_window_set_icon(GTK_WINDOW(window),gpb);
  g_object_unref(gpb);

  gtk_signal_connect(GTK_OBJECT(window), "delete_event",GTK_SIGNAL_FUNC(eventDelete),NULL);
  gtk_signal_connect(GTK_OBJECT(window), "destroy",GTK_SIGNAL_FUNC(eventDestroy),NULL);

  box = gtk_vbox_new(FALSE,0);
  heading = gtk_label_new(less[cless][0]);
  line = gtk_label_new(less[cless][cline]);
  gtk_label_set_justify(GTK_LABEL(line),GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment(GTK_MISC(line),0,0);
  entry = gtk_entry_new();

  gtk_signal_connect(GTK_OBJECT(entry),"changed",GTK_SIGNAL_FUNC(tchanged),NULL);

  gtk_box_pack_start(GTK_BOX(box),heading,FALSE,FALSE,0);
  gtk_box_pack_start(GTK_BOX(box),line,FALSE,FALSE,0);
  gtk_box_pack_start(GTK_BOX(box),entry,FALSE,FALSE,0);

  im = gtk_image_new_from_file("keycaps.png");
  gtk_box_pack_start(GTK_BOX(box),im,FALSE,FALSE,0);


  gtk_container_add(GTK_CONTAINER(window),box);
  gtk_widget_show_all(window);

  curline = less[cless][cline];
  started = 0;

  gtk_main();
  return 0;
}  

/*
  Detect input and act on it (i.e. check if it's right)
*/ 
gint tchanged(GtkWidget *widget, gpointer data) {
  gchar *lin = strdup(gtk_entry_get_text(GTK_ENTRY(widget)));
  char wo[300];
  if (started == 0&&strcmp(lin,"")) {
    started = time(NULL);
  }
  if (strncmp(lin,curline,strlen(lin))) {
    g_print("\7");
    lin[strlen(lin)-1] = '\0';
    gtk_entry_set_text(GTK_ENTRY(widget),lin);
    errors++;
  }
  if (strlen(lin) == strlen(curline)) {
    length += time(NULL) - started;
    started = 0;
    cline++;
    if (cline >= linless[cless]) {
      GtkWidget *dialog;
      cless++;
      cline = 1;
      if ((errors*100/letters>=12)||(letters*60/length < 50)) {
        dialog = gtk_message_dialog_new(GTK_WINDOW(window),GTK_DIALOG_MODAL,GTK_MESSAGE_INFO,GTK_BUTTONS_OK,
                   "Time: %d seconds\n"
                   "Lpm:  %d\n"
                   "Errors:  %d\n"
                   "Error Percentage:  %d\n"
                   "Your error percentage was too high; you'll have to redo the lesson",
                   length,letters*60/length,errors,errors*100/letters);
       cless--;
       gtk_dialog_run(GTK_DIALOG(dialog));
       gtk_widget_destroy(dialog);
     } else if (cless == numless) {
        dialog = gtk_message_dialog_new(GTK_WINDOW(window),GTK_DIALOG_MODAL,GTK_MESSAGE_INFO,GTK_BUTTONS_OK,
                   "Time: %d seconds\n"
                   "Lpm:  %d\n"
                   "Errors:  %d\n"
                   "Error Percentage:  %d\n"
                   "Wow, you finished this typing course!\n"
                   "It's recommended that you start over,\n"
                   "to make sure you know everything",
                   length,letters*60/length,errors,errors*100/letters);
       gtk_dialog_run(GTK_DIALOG(dialog));
       gtk_widget_destroy(dialog);
       unlink(PROGFILE);
       gtk_main_quit();
       return 0;
     } else {
       dialog = gtk_message_dialog_new(GTK_WINDOW(window),GTK_DIALOG_MODAL,GTK_MESSAGE_INFO,GTK_BUTTONS_OK,
                   "Time: %d seconds\n"
                   "Lpm:  %d\n"
                   "Errors:  %d\n"
                   "Error Percentage:  %d",
                   length,letters*60/length,errors,errors*100/letters);
       gtk_dialog_run(GTK_DIALOG(dialog));
       gtk_widget_destroy(dialog);
     }

      letters = 0;
      length = 0;
      errors = 0;
      gtk_label_set_text(GTK_LABEL(heading),less[cless][0]);
    }
    curline = less[cless][cline];
    letters += strlen(curline);
    gtk_label_set_text(GTK_LABEL(line),curline);
    gtk_label_set_justify(GTK_LABEL(line),GTK_JUSTIFY_LEFT);
    gtk_entry_set_text(GTK_ENTRY(widget),"");
    sprintf(wo,"ABCD Dvorak - %d/%d %d/%d",cless,numless,cline,linless[cless]);
    gtk_window_set_title(GTK_WINDOW(window),wo);
  }
  return FALSE;
}

gint eventDelete(GtkWidget *widget, GdkEvent *event,gpointer data) {
  return FALSE;
}

gint eventDestroy(GtkWidget *widget, GdkEvent *event,gpointer data) {
  FILE *fil = fopen(PROGFILE,"w");
  fprintf(fil,"%d %d %d %d %d",cless, cline, length, errors, letters);
  fclose(fil);
  gtk_main_quit();
  return 0;
}


   
 